/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public 
 * License version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Initial Developer of the Original Code is R.V.Guha. Portions
 * created by R.V.Guha are copyright (c) 2000 R.V.Guha. All Rights
 * Reserved.
 *
 * Contributors: Ivan Kurmanov, Jasper van de Gronde (win32 port)
 */

#ifdef WIN32
#include <winsock2.h>
#include <windows.h>
#endif
#include "utils.h"

// for testing purposes
#include <stdio.h>

char* 
getMem (size_t l) {
  char* ans = malloc(l);
  if (ans) bzero(ans, l);
  return ans;
}

void
freeMem (const char* mem) {
  free((void*)mem); 
}

char* 
trimLine (char* line) {
  size_t len = strlen(line);
  size_t index = len-1;
  while (index > 0) {
    if (isspace(line[index])) {
      line[index] = 0;
      index--;
    } else break;
  }
  index = 0;
  while (index < len) {
    if (isspace(line[index])) {
      index++;
    } else break;
  }
  return &line[index];
}

int
nullString (const char* line, int len) {
  int n = 0;
  while (n < len) {
    if (!isspace(line[n])) return 0;
    n++;
  } 
  return 1;
}


int
startsWith (const char* str, const char* prefix) {
  size_t l1 = strlen(str);
  size_t l2 = strlen(prefix);
  if (l1 >= l2) {
    return (strncasecmp(str, prefix, l2) == 0);
  } else return 0;
}


/*****   starts with forgiving white space    ******/
int
startsWith_forgiving_ws (const char* actual_str, const char* prefix) {
  const char * str = actual_str;
  while( isspace( str[0] ) ) {
    str++;
  }

  {
    size_t l1 = strlen(str);
    size_t l2 = strlen(prefix);
    if (l1 >= l2) {
      return (strncasecmp(str, prefix, l2) == 0);
    } else return 0;
  }
}

int
endsWith (const char* str, const char* suffix) {
  size_t l1 = strlen(str);
  size_t l2 = strlen(suffix);

  if (l1 >= l2) {
    return (strncasecmp(&str[l1-l2], suffix, l2) == 0);
  } else return 0;
}

#ifdef WIN32
int
writen(SOCKET socket, char *ptr, int nbytes) {
  int	nleft, nwritten;
  
  nleft = nbytes;
  while (nleft > 0) {
    nwritten = send(socket, ptr, nleft, 0);
    if (nwritten <= 0)
      return(nwritten);		/* error */
    
    nleft -= nwritten;
    ptr   += nwritten;
  }
  return(nbytes - nleft);
}
#else
int
writen(int fd, char *ptr, int nbytes) {
  int	nleft, nwritten;
  
  nleft = nbytes;
  while (nleft > 0) {
    nwritten = write(fd, ptr, nleft);
    if (nwritten <= 0)
      return(nwritten);		/* error */
    
    nleft -= nwritten;
    ptr   += nwritten;
  }
  return(nbytes - nleft);
}
#endif

/*
 * Read a line from a descriptor.  Read the line one byte at a time,
 * looking for the newline.  We store the newline in the buffer,
 * then follow it with a null (the same as fgets(3)).
 * We return the number of characters up to, but not including,
 * the null (the same as strlen(3)).
 */

#ifdef WIN32
int
readline(SOCKET socket, char *ptr_p, int maxlen) {
  int	n, rc;
  char	c;
  char *ptr = ptr_p;
  
  for (n = 1; n < maxlen; n++) {
    if ( (rc = recv(socket, &c, 1, 0)) == 1) {
      *ptr++ = c;
      /* Jasper van de Gronde
       * Made this check for both CR and LF.
       * TODO: check for CRLF.
       */
      if (c == '\x0A' /*LF*/ || c == '\x0D' /*CR*/)         
          break;
    } else if (rc == 0) {
      if (n == 1)
        return(0);	/* EOF, no data read */
      else
        break;		/* EOF, some data was read */
    } else {
      *ptr = 0;
      printf("Line: %s\n", ptr_p);
      return(-1);	/* error */
    }
  }
  
  *ptr = 0;
  return(n);
}
#else
int
readline(int fd, char *ptr, int maxlen) {
  int	n, rc;
  char	c;
  
  for (n = 1; n < maxlen; n++) {
    if ( (rc = read(fd, &c, 1)) == 1) {
      *ptr++ = c;
      if (c == NEWLINE_CHAR)         
        break;
    } else if (rc == 0) {
      if (n == 1)
        return(0);	/* EOF, no data read */
      else
        break;		/* EOF, some data was read */
    } else
      return(-1);	/* error */
  }
  
  *ptr = 0;
  return(n);
}
#endif

/* Read upto maxlen bytes off the fd */

#ifdef WIN32
int
readn(SOCKET socket, char *ptr, int maxlen) {
  int	n, rc;
//  char	c;
  
  bzero(ptr, maxlen);
  for (n = 1; n < maxlen; n++) {
    if ((rc = recv(socket, ptr, 1, 0)) == 1) {
      /*      *ptr++ = c; */
    } else if (rc == 0) {
      if (n == 1)
        return(0);	/* EOF, no data read */
      else
        break;		/* EOF, some data was read */
    } else
      return(-1);	/* error */
  }
  
  /*  *ptr = 0; */
  return(n);
}
#else
int
readn(int fd, char *ptr, int maxlen) {
  int	n, rc;
//  char	c;
  
  bzero(ptr, maxlen);
  for (n = 1; n < maxlen; n++) {
    if ((rc = read(fd, ptr, 1)) == 1) {
      /*      *ptr++ = c; */
    } else if (rc == 0) {
      if (n == 1)
        return(0);	/* EOF, no data read */
      else
        break;		/* EOF, some data was read */
    } else
      return(-1);	/* error */
  }
  
  /*  *ptr = 0; */
  return(n);
}
#endif

/* Jasper van de Gronde
 * I made this to calculate how many digits a number will take,
 * so this can be used in calculating how much memory must be
 * reserved when converting the number to a string.
 */
int numDigits(int n)
{
    int digits;
    if ( n < 0 ) {
        digits = 2; // take the '-' into account
        n = -n;
    } else {
        digits = 1;
    }
    while(n > 9) {
        n /= 10;
        ++digits;
    }
    return digits;
}

/* Jasper van de Gronde
 * Duplicates the string starting at begin and ending at end.
 */
char* strduprange(const char* begin, const char* end) {
  const size_t size = end-begin;
  char* s = getMem(size+1);
  if ( s ) memcpy(s, begin, size);
  return s;
}